interface PlainMessage {
    kind: "plain";
    message: string;
}
interface CommandParameter {
    key: string;
    value: string[];
}
interface CommandMessage {
    kind: "command";
    action: string;
    actionTargets: string[];
    parameters?: Record<string, string[]>;
}
declare type MatchedCommandMessage = CommandMessage & {
    matched: string;
};
declare type ParsedCommandMessage = PlainMessage | MatchedCommandMessage;
interface MessageItem {
    /**
     * @example get release latest
     */
    command: string;
    /**
     * Command Description
     */
    description: string;
}
interface ParsedInputCommand extends Omit<CommandMessage, "kind"> {
    command: string;
}
interface ParserInitialize {
    messageItems: MessageItem[];
}
declare class Parser {
    private readonly params;
    private readonly commandActionRegExp;
    private readonly validActions;
    private readonly parsedInputCommands;
    constructor(params: ParserInitialize);
    /**
     * Parses a string into defined commands
     * If no match, returns input message without parsing
     */
    parse(message: string): PlainMessage | MatchedCommandMessage;
    /**
     * Verify the existence of duplicate commands
     */
    private validateDuplicateCommandExists;
    /**
     * Array of valid action strings for this Parser
     *
     * Extract the leading `\w+` from "get hoge", "create fuga", etc. and convert them into an array ["get", "create"].
     */
    private getValidActions;
    /**
     * Parses input commands and converts them into programmable objects
     */
    private getParsedInputCommands;
    /**
     * Regular expression that breaks down the string before the parameters string into action and actionTargets
     */
    private createCommandActionRegExp;
    /**
     * convert input strings (`a:b:c` or `version:v0.1.2`) to key value format objects
     * for `paramsStrings` extracted by `Utils.splitParameters
     */
    private createParsedParameters;
    /**
     * Returns an array matching the commands defined during Parser initialization
     */
    private getMatchedParsedInputCommands;
    /**
     * Parsing input messages
     *
     * @param message
     * @param type "system" | "user" Use "system" for internal use and "user" for external (public) use
     * @returns
     */
    private parseMessage;
    /**
     * Object for strings that do not match the defined command
     */
    private createPlainMessage;
}

export { CommandMessage, CommandParameter, MatchedCommandMessage, MessageItem, ParsedCommandMessage, ParsedInputCommand, Parser, ParserInitialize, PlainMessage };
